package de.fzj.unicore.uas.functional;

import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.ApplicationType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.uas.security.ProxyCertificate;
import de.fzj.unicore.uas.testsuite.AbstractJobRun;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;


/**
 * TODO --- need to run stuff like this against an installation that is proxy enabled...
 */
public class RunBasicTestsOfProxyForTLS extends AbstractJobRun{

	String url;
	EndpointReferenceType tssepr;

	final String keyLocation="target/test/keystore.jks";

	@Override
	public void testRunJob()throws Exception{
		//enable proxy 
		Kernel.getKernel().setProperty(ISecurityProperties.UAS_AUTHORISER_PROXY_SUPPORT, "true");
		File test=new File("target/test");
		if(!test.exists()){
			boolean OK=new File("target/test").mkdirs();
			if(!OK){
				throw new IOException("Can't create directory 'target/test'");
			}
		}
		//setup client keystore
		ProxyCertificate proxy=ProxyCertificate.getInstance(Kernel.getKernel().getSecurityProperties(), false, 3600, 512);
		proxy.writeJKS(keyLocation);

		super.testRunJob();
	}

	@Override
	protected void runJob(TSSClient tss) throws Exception{
		//nop
	}

	@Override
	protected ISecurityProperties getSecurityProperties(){
		ISecurityProperties sec=Kernel.getKernel().getSecurityProperties().clone();
		sec.setProperty(ISecurityProperties.WSRF_SSL_KEYSTORE, keyLocation);
		sec.setProperty(ISecurityProperties.WSRF_SSL_KEYALIAS, "mykey");
		sec.setProperty(ISecurityProperties.WSRF_SSL_KEYTYPE, "jks");
		sec.setProperty(ISecurityProperties.WSRF_SSL_KEYPASS, "unicore");
		return sec;
	}

	@Override
	protected JobDefinitionDocument getJob() {
		JobDefinitionDocument jdd=JobDefinitionDocument.Factory.newInstance();
		ApplicationType app=jdd.addNewJobDefinition().addNewJobDescription().addNewApplication();
		app.setApplicationName("Date");
		app.setApplicationVersion("1.0");
		return jdd;
	}

	@Override
	protected void beforeStart(JobClient job)throws Exception{
		System.out.println("Job submitted at: "+job.getSubmissionTime());
		job.getUspaceClient().getImport("test.txt").write("import test".getBytes());
		job.getUspaceClient().getImport("folder/test.txt").write("import test in subfolder".getBytes());
	}

	@Override
	protected void onFinish(JobClient job) throws Exception{
		String out=getStdout(job);
		assertTrue(out!=null && out.length()>0);
		job.destroy();
	}


}
