package de.fzj.unicore.uas.functional;

import java.io.ByteArrayInputStream;
import java.util.List;

import junit.framework.TestCase;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.TargetSystem;
import de.fzj.unicore.uas.UAS;
import de.fzj.unicore.uas.client.BaseUASClient;
import de.fzj.unicore.wsrflite.Kernel;
import de.fzj.unicore.wsrflite.security.ISecurityProperties;
import de.fzj.unicore.wsrflite.security.UASSecurityProperties;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;

/**
 * (part of) a test suite for UNICORE 6
 * (uses UNICORE/X through a Gateway)
 */
public class SecuredTestSuite extends TestCase{

	private static String url="https://localhost:8080/DEMO-SITE/services/";

	//epr of the target system to use
	private static EndpointReferenceType tsEPR;

	private static EndpointReferenceType regEPR;

	private static boolean firstTime=true;

	protected void setUp(){
		if(firstTime){
			tsEPR=EndpointReferenceType.Factory.newInstance();
			tsEPR.addNewAddress().setStringValue(url+UAS.TSS+"?res=default_targetsystem");
			WSServerUtilities.addUGSRefparamToEpr(tsEPR);
			System.out.println("Using TSS at "+tsEPR);

			regEPR=EndpointReferenceType.Factory.newInstance();
			regEPR.addNewAddress().setStringValue(url+UAS.REG+"?res=default_registry");
			WSServerUtilities.addUGSRefparamToEpr(regEPR);
				System.out.println("Using registry at "+regEPR);
				firstTime=false;
			}
		}
		
		protected String getUrl(){return url;}
		
		//security settings...
		private static String sprops=
				ISecurityProperties.WSRF_SSL + "=true"+"\n"+
				ISecurityProperties.WSRF_SSL_CLIENTAUTH + "=false" +"\n"+
				ISecurityProperties.WSRF_SSL_KEYSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
				ISecurityProperties.WSRF_SSL_KEYTYPE + "=JKS"+"\n"+
				ISecurityProperties.WSRF_SSL_KEYPASS + "=the!user"+"\n"+
				
				ISecurityProperties.WSRF_SSL_TRUSTSTORE + "=src/main/certs/user-keystore.jks" +"\n"+
				ISecurityProperties.WSRF_SSL_TRUSTTYPE + "=JKS"+"\n"+
				ISecurityProperties.WSRF_SSL_TRUSTPASS + "=the!user"+"\n"
				;
				
			
		protected ISecurityProperties getSecurityProperties() throws Exception {
			UASSecurityProperties uas= new UASSecurityProperties(Kernel.getKernel().getProperties(), 
					new ByteArrayInputStream(sprops.getBytes()));
			uas.setSignMessage(true);
			return uas;
		}

		

		public void test1() throws Exception {
			BaseUASClient wsrfClient=new BaseUASClient(getUrl()+UAS.TSS,tsEPR,getSecurityProperties());
			
			System.out.println();
			System.out.println("***** Target System Properties ****");
			System.out.println();
			
			System.out.println(wsrfClient.getResourcePropertyDocument());
		}
		

		public void test2() throws Exception {
			BaseUASClient wsrfClient=new BaseUASClient(getUrl()+UAS.REG,regEPR,getSecurityProperties());
			
			System.out.println();
			System.out.println("***** Registry Properties ****");
			System.out.println();
			
			System.out.println(wsrfClient.getResourcePropertyDocument());
		}

		
		public void testServiceDiscovery(){
			try{
				RegistryClient reg=new RegistryClient(getUrl()+UAS.REG,regEPR,getSecurityProperties());
				List<EndpointReferenceType> l=reg.listServices(TargetSystem.TSS_PORT);
				for(EndpointReferenceType e: l)System.out.println(e);
			}catch(Exception e){
				e.printStackTrace();
				fail();
			}
		}
		

}
