package eu.unicore.genii.security;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;

import org.apache.log4j.Logger;

/**
 * helper code to create a truststore from pem files
 * 
 * @author schuller
 */
public class TruststoreUtil {
	
	private static int counter=0;
	

	public static KeyStore loadTruststoreFromPemFile(String name) throws IOException,NoSuchAlgorithmException,CertificateException,KeyStoreException {
		KeyStore trustStore= KeyStore.getInstance("jks");
		trustStore.load(null, "unicore".toCharArray());
		loadPemFile(name,trustStore);
		return trustStore;
	}


	public static KeyStore loadTruststoreFromDirectory(String name) throws IOException,NoSuchAlgorithmException,CertificateException,KeyStoreException {
		KeyStore trustStore= KeyStore.getInstance("jks");
		trustStore.load(null, "unicore".toCharArray());
		File directory=new File(name);
		File[] pems=directory.listFiles(new FilenameFilter(){
			public boolean accept(File f, String name){
				return name.endsWith(".pem") || name.endsWith(".cert");
			}
		});
		if(pems!= null)
		{
			for(File pem: pems){
				loadPemFile(pem.getAbsolutePath(),trustStore);
			}
		}
		return trustStore;
	}

	//load all the certs from the given pem file and put them as trusted certs into the keystore
	public static void loadPemFile(String name, KeyStore ks)throws IOException, KeyStoreException, CertificateException{
		CertificateFactory cf=CertificateFactory.getInstance("X.509");
		BufferedInputStream bis=new BufferedInputStream(new FileInputStream(name));
		try{
			while(true){
				X509Certificate cert=(X509Certificate)cf.generateCertificate(bis);
				String dn=cert.getSubjectX500Principal().toString();
				ks.setCertificateEntry("trusted-"+counter, cert);
				//check if we have more data...
				bis.mark(1);
				if(bis.read()==-1)break;
				bis.reset();
				counter++;
			}
		}
		finally{
			try{bis.close();}catch(IOException io){};
		}
	}
	
}
